@@ -18,9 +18,10 @@ import datetime as dt
 import re
 import json
 from dataclasses import dataclass
+from enum import Enum
 
 import flask
-import threading
+import threading  # for Local and Lock
 
 # for local use & forwarding
 from flask import Response, request
@@ -88,10 +89,11 @@ def _typeof(p: inspect.Parameter):
 class _Pool:
     """Thread-safe pool of something, created on demand."""
 
-    def __init__(self, fun: Callable[[int], Any]):
+    def __init__(self, fun: Callable[[int], Any], max_size: int = 0):
         self._lock = threading.Lock()
         self._fun = fun
         self._nobjs = 0
+        self._max_size = max_size
         self._available: Set[Any] = set()
         self._using: Set[Any] = set()
 
@@ -100,19 +102,21 @@ class _Pool:
         with self._lock:
             try:
                 obj = self._available.pop()
+                self._using.add(obj)
             except KeyError:
+                if self._max_size and self._nobjs >= self._max_size:
+                    raise Exception(f"object pool max size reached ({self._max_size})")
                 log.debug(f"creating new obj with {self._fun}")
                 obj = self._fun(self._nobjs)
                 self._nobjs += 1
-            finally:
                 self._using.add(obj)
             return obj
 
     def ret(self, obj):
         """Return object to pool."""
         with self._lock:
-           self._using.remove(obj)
-           self._available.add(obj)
+            self._using.remove(obj)
+            self._available.add(obj)
 
 
 class Reference:
@@ -132,18 +136,41 @@ class Reference:
     initialized directly or by providing a generation functions.
     The generation function is called on demand in each thread automatically.
     """
+
     class Local(object):
+        """
+        Dumb storage class for shared mode.
+        """
         pass
 
-    def __init__(self, obj: Any = None, set_name: str = "set", fun: Optional[Callable] = None, pool: bool = False):
+    class Mode(Enum):
+        """
+        Define granularity of object sharing
+
+        - SHARED: only one object, which should be thread safe.
+        - THREAD: per-thread object, generated by a function.
+        - VERSATILE: sub-thread-level object (eg greenlet), generated by a
+          function.
+        """
+        AUTO = 0
+        SHARED = 1
+        THREAD = 2
+        VERSATILE = 3
+
+    def __init__(self, obj: Any = None, set_name: str = "set", fun: Optional[Callable] = None, max_size: Optional[int] = None, mode: Mode = Mode.AUTO):
         """Constructor parameters:
 
         - set_name: provide another prefix for the "set" functions.
         - obj: object to be wrapped, can also be provided later.
-        - fun: function to generated a per-thread wrapped object.
-        - pool: whether to use a pool on top of fun.
+        - fun: function to generated a per-thread/or-whatever wrapped object.
+        - max_size: pool maximum size, 0 for unlimited, None for no pooling.
+        - mode: level of sharing, default is to chose between SHARED and THREAD.
         """
-        self._pool = pool
+        # mode encodes the expected object unicity or multiplicity
+        self._mode = Reference.Mode.SHARED if mode == Reference.Mode.AUTO and obj else \
+                     Reference.Mode.THREAD if mode == Reference.Mode.AUTO and fun else \
+                     mode
+        self._pool_max_size = max_size
         self._set(obj=obj, fun=fun, mandatory=False)
         if set_name and set_name != "_set":
             setattr(self, set_name, self._set)
@@ -153,7 +180,9 @@ class Reference:
     def _set_obj(self, obj):
         """Set current wrapped object."""
         log.debug(f"setting reference to {obj} ({type(obj)})")
+        self._mode = Reference.Mode.SHARED
         self._fun = None
+        self._pool = None
         self._nobjs = 1
         self._local = self.Local()
         self._local.obj = obj
@@ -161,11 +190,18 @@ class Reference:
 
     def _set_fun(self, fun: Callable[[int], Any]):
         """Set current wrapped object generation function."""
+        if self._mode == Reference.Mode.AUTO:
+            self._mode = Reference.Mode.THREAD
+        assert self._mode in (Reference.Mode.THREAD, Reference.Mode.VERSATILE)
         self._fun = fun
-        if self._pool:
-            self._pool_set = _Pool(fun)
+        self._pool = _Pool(fun, self._pool_max_size) \
+            if self._pool_max_size is not None else None
         self._nobjs = 0
-        self._local = threading.local()
+        if self._mode == Reference.Mode.THREAD:
+            self._local = threading.local()
+        else:  # Reference.Mode.VERSATILE
+            import werkzeug
+            self._local = werkzeug.local.Local()
         return self
 
     def _set(self, obj: Any = None, fun: Optional[Callable[[int], Any]] = None, mandatory=True):
@@ -182,19 +218,21 @@ class Reference:
     def _get_obj(self):
         """Get current wrapped object."""
         if self._fun and not hasattr(self._local, "obj"):
-            if self._pool:
-                self._local.obj = self._pool_set.get()
-                self._nobjs = self._pool_set._nobjs  # FIXME
+            if self._pool_max_size is not None:
+                self._local.obj = self._pool.get()
+                self._nobjs = self._pool._nobjs  # FIXME
             else:
                 self._local.obj = self._fun(self._nobjs)
-                self._nobjs += 1
+                self._nobjs += 1  # FIXME?
         return self._local.obj
 
+    # FIXME how to do that automatically when the thread/whatever ends?
     def _ret_obj(self):
         """Return current wrapped object to internal pool."""
-        assert self._pool
-        self._pool_set.ret(self._local.obj)
-        delattr(self._local, "obj")
+        if self._pool_max_size is not None:
+            self._pool.ret(self._local.obj)
+            delattr(self._local, "obj")
+        # else just ignore
 
     def __getattr__(self, item):
         """Forward everything unknown to contained object."""
@@ -1075,8 +1113,7 @@ class FlaskSimpleAuth:
                 token = request.cookies[self._name] \
                     if self._name in request.cookies else None
             elif self._carrier == "param":
-                params = self._params()
-                token = params.get(self._name, None)
+                token = self._params().get(self._name, None)
             else:
                 assert self._carrier == "header" and self._name
                 token = request.headers.get(self._name, None)
