@@ -13,17 +13,13 @@ log = logging.getLogger("app")
 #
 import FlaskSimpleAuth as fsa
 from FlaskSimpleAuth import Flask, jsonify, ANY, ALL, NONE, path, string
-app = Flask("Test")
+app = Flask("Test", debug=True)
 
 #
 # AUTH
 #
 app.config.update(
     FSA_AUTH = "fake",
-    FSA_MODE = "always",
-    FSA_SKIP_PATH = (r"/(register|required|_|my|json)",
-                     r"/(add|div|mul|sub|type|params|any|mis[12]|nogo|one)",
-                     r"/(infer|superid|cplx|bool|mail|path|string|auth|f2|myint)"),
     FSA_GET_USER_PASS = get_user_pass,
     FSA_USER_IN_GROUP = user_in_group,
     FSA_LOGGING_LEVEL = logging.DEBUG,
@@ -34,7 +30,7 @@ app.config.update(
 def check_users_perms(login: str, val: str, mode):
     return login in (val, "dad") if val in UP else None
 
-app.register_object_perms("users", check_users_perms)
+app.object_perms("users", check_users_perms)
 
 from SubApp import subapp
 app.register_blueprint(subapp, url_prefix="/b1")
@@ -236,7 +232,7 @@ class my_int:
         i.val = int(s)
         return i
 
-app.register_cast(my_int, my_int.str_to_my_int)
+app.cast(my_int, my_int.str_to_my_int)
 
 @app.get("/myint/<i>", authorize=ANY)
 def get_myint_i(i: my_int):
